import { Constant } from '../../decorators/constant';

@Constant('urlUtils', true)

/**
 * @class UrlUtils
 *
 * Utilities for URLs
 */
export class UrlUtils {
  /**
   * Encode and add a param to a url
   *
   * @example
   * ```ts
   * urlUtils.addParam('foo.com', 'foo', 'b a r');
   * // returns 'foo.com?foo=b%20a%20r'
   *
   * utils.addParam('foo.com?already=hasParams', 'foo', 'b a r');
   * // returns 'foo.com?already=hasParams&foo=b%20a%20r'
   * ```
   *
   * @param  {string} url   Url to add param to
   * @param  {string} name  Param key
   * @param  {string} value Param value
   * @return {string}       Newly const ructed url
   */
  addParam(url: string, name: string, value: string): string {
    value = this.encodeURIComponent(value);

    if (url.indexOf('?') < 0) {
      url += '?';
    } else {
      url += '&';
    }

    url += name + '=' + value;
    return url;
  }

  /**
   * Encode a URI compoent angular style
   */
  encodeURIComponent(str: string) {
    let uri;
    try {
      uri = decodeURIComponent(str);
    } catch(err) {
      uri = str;
    }

    return encodeURIComponent(uri)
      .replace(/%40/gi, '@')
      .replace(/%3A/gi, ':')
      .replace(/%3B/gi, ';')
      .replace(/%24/g, '$')
      .replace(/%2C/gi, ',')
      .replace(/%20/g, '+');
  }

  /**
   * Parse query params from a URL string
   */
  getQueryParams(url: string) {
    const queryString = url.split('?')[1] || url;
    const queryParams = {};

    queryString
      .split('&')
      .forEach((keyVal: string) => {
        const [key, value] = keyVal.split('=');
        const currentValue = queryParams[key];

        if (currentValue) {
          if (!_.isArray(currentValue)) {
            queryParams[key] = [currentValue];
          }

          queryParams[key].push(value);
        } else {
          queryParams[key] = value;
        }
      });

    return queryParams;
  }

  /**
   * Encodes and adds a map of param names/values sorting them
   * alphabetically for consistent cache keys
   *
   * @example
   * ```ts
   * urlUtils.addParams('foo.com', {
   * 	 foo: 'bar',
   * 	 list: ['a', 'b', 'c']
   * });
   * // returns 'foo.com?foo=bar&list=a&list=b&list=c'
   * ```
   *
   * @param  {string} url      Start URL
   * @param  {any}    paramMap Map of params to add
   * @return {string}          New URL
   */
  addParams(url: string, paramMap: any): string {
    Object
      .keys(paramMap)
      .sort()
      .forEach((name) => {
        const value = paramMap[name];

        if (_.isArray(value)) {
          for (const item of value) {
            url = this.addParam(url, name, item);
          }
        } else {
          url = this.addParam(url, name, value);
        }
      });

    return url;
  }

  /**
   * Generates the absolute path for a url when
   * it is within a cordova app. Necessary because
   * lazy loading/webpack messes with how cordova usually
   * does this automatically.
  */
  getAbsolutePathForCordova(url = '') {
    let path = window.location.pathname.split('/');
    path.pop();
    return path.join('/') + '/' + url;
  }

  parseUrl(url: string) {
    // returns an anchor tag from which you can access
    // protocol, hostname, port, pathname, search, hash
    if (!url) {
      return;
    }

    const parser = document.createElement('a');
    parser.href = url;

    const parsedUrl = {
      href: url,
      protocol: parser.protocol,
      host: parser.hostname,
      port: parser.port,
      path: (() => {
        if (/^\//.test(parser.pathname)) {
          return parser.pathname;
        } else {
          // the parsed pathname on IE doesn't contain a preceding /
          // we need to manually add it or our generated hrefs will be relative
          return `/${parser.pathname}`;
        }
      })(),
      search: parser.search,
      query: (() => {
        const query: any = {};
        const queryParams = parser.search.substr(1).split('&');
        queryParams.forEach((param) => {
          const pair = param.split('=');
          query[pair[0]] = pair[1];
        })

        return query;
      })(),
    };

    return parsedUrl;
  }
}
