import '@host';

import { Service }           from '../../decorators/service';
import { inject }            from '../../decorators/inject';
import { IUser }             from '../../../../common/interfaces/user';
import 'angular-easyfb';
import './login-config';
const gapi = require('./gapi-client');

@Service('user')

/**
 * @class User
 *
 */
export class User {
  @inject
  private $q: angular.IQService;

  @inject
  $rootScope: angular.IRootScopeService;

  @inject
  private $cookies: angular.cookies.ICookiesService;

  @inject
  private $http: angular.IHttpService;

  @inject
  private config: any;

  @inject
  private ezfb: any;

  referrer: string;
  data: IUser;

  constructor() {
    this.data = this.$cookies.getObject('userData');
    if (!this.data) {
      return;
    } else if (this.data.loginTimestamp && this.data.loginToken) {
      this.data.$isLoggedIn = true;
    }
  }

  get isLoggedIn() {
    return Boolean(this.data && this.data.$isLoggedIn);
  }

  signup(username: string, password: string, options?: any) {
    const deferred = this.$q.defer();

    if (this.data && this.data.$isLoggedIn) {
      deferred.resolve();
      return deferred.promise.then(() => {
        return this.data;
      });
    }

    const data: any = {
      password: password
    };

    if (this.isValidEmail(username)) {
      data.email = username;
    } else {
      deferred.reject({
        errorMessage: 'Invalid email address'
      });
      return deferred.promise;
    }

    const params = {
      format: 'json',
      includeCookie: true,
      track: false,
      ugcCounts: true
    };

    const signUpUrl = `${this.config.secureApiRoot}/api/v2/users`;

    return this
      .$http
      .post(signUpUrl, data, { params: params })
      .then((response: any) => {
        return this
          .checkLoginApiResponse(response)
          .then((loginResponse: any) => {
            this.successfulAuth(loginResponse);
            return loginResponse;
          })
          .catch((err: any) => {
            deferred.reject(err);
            return this.$q.reject(err);
          });
      });
  }

  login(username: string, password: string, options?: any) {
    const deferred = this.$q.defer();

    if (this.data && this.data.$isLoggedIn) {
      deferred.resolve();
      return deferred.promise.then(() => {
        return this.data;
      });
    }

    const data: any = {
      password: password
    };

    if (this.isValidEmail(username)) {
      data.email = username;
    } else {
      data.handle = username;
    }

    const params = {
      format: 'json',
      includeCookie: true,
      track: false,
      ugcCounts: true
    };

    const loginApiUrl = `${this.config.secureApiRoot}/api/v2/login`;

    return this
      .$http
      .post(loginApiUrl, data, {params: params})
      .then((response: any) => {
        return this
          .checkLoginApiResponse(response)
          .then((loginResponse: any) => {
            this.successfulAuth(loginResponse);
            return loginResponse;
          })
          .catch((err: any) => {
            deferred.reject(err);
            return this.$q.reject(err);
          });
      });
  }

  facebookLogin() {
    const deferred = this.$q.defer();

    this
      .ezfb
      .login((response: any) => {
        if (!response || !response.authResponse || !response.authResponse.accessToken) {
          const error = new Error('Something went wrong, please try again.');
          deferred.reject(error);
          return this.$q.reject(error);
        }

        const data = {
          accessToken: response.authResponse.accessToken
        };

        const params = {
          format: 'json',
          includeCookie: true,
          track: false
        };

        return this
          .$http
          .post(`${this.config.secureApiRoot}/api/v2/login/facebook`, data, { params })
          .then((loginResponse: any) => {
            return this
              .checkLoginApiResponse(loginResponse)
              .then((apiResponse: any) => {
                this.successfulAuth(apiResponse);
                deferred.resolve(apiResponse);
                return apiResponse;
              })
              .catch((err: any) => {
                deferred.reject(err);
                return this.$q.reject(err);
              });
          });
      }, { scope: 'email' });

    return deferred.promise;
  }

  googleLogin() {
    const self = this;
    const deferred = this.$q.defer();
    gapi.auth.authorize({
      client_id: '334881074383.apps.googleusercontent.com',
      scope: 'https://www.googleapis.com/auth/plus.login https://www.googleapis.com/auth/userinfo.email',
      fetch_basic_profile: true,
      response_type: 'code token'
    }, (authResult: any) => {
      if (!authResult || authResult.error || authResult instanceof Error) {
        deferred.reject(authResult);
        return this.$q.reject(authResult);
      } else {
        const data: any = {
          code: authResult.code
        };

        const params = {
          format: 'json',
          includeCookie: true,
          track: false,
          ugcCounts: true
        };

        self
          .$http
          .post(`${this.config.secureApiRoot}/api/v2/login/google`, data, { params })
          .then((apiResponse: any) => {
            self.successfulAuth(apiResponse);
            deferred.resolve(apiResponse);
            return apiResponse;
          })
          .catch((err: any) => {
            deferred.reject(err);
            return this.$q.reject(err);
          });
      }
    });

    return deferred.promise;
  }

  private checkLoginApiResponse(response: any) {
    const deferred = this.$q.defer();

    if (!response || response.status >= 400 || !response.data) {
      deferred.reject(response);
    } else {
      deferred.resolve(response);
    }

    return deferred.promise;
  }

  private successfulAuth(response: any) {
    const userResponse: IUser = response.data.user;
    _.extend(userResponse, {
      $isLoggedIn: true,
      loginTimestamp: response.data.loginTimestamp,
      loginToken:  response.data.loginToken
    });
    this.updateCookie(userResponse);
    this.$rootScope.$broadcast('user.shopstyleAuthSuccess', this.data);
  }

  private updateCookie(userData: any) {
    // TODO: should we store in localstorage?
    //
    const cookieWhitelist = [
      'apiKey',
      'authLevel',
      'email',
      'handle',
      'id',
      'authentication',
      'type',
      'loginTimestamp',
      'loginToken',
      'paypalEmail',
      'paypalFirstName',
      'paypalLastName'
    ];

    if (this.data) {
      _.extend(this.data, userData);
    } else {
      this.data = userData;
    }

    this.$cookies.putObject(
      'userData',
      _.pick(this.data, cookieWhitelist),
      { domain: this.config.tld }
    );
  }

  put(newFields: any): ng.IPromise<User> {
    const userUrl = `${this.config.apiRoot}/api/v2/users/${this.data.id}`;
    return this.$http.put(userUrl, newFields).then(() => {
      this.updateCookie(newFields);
      return this;
    });
  }

  logOut(noReload?: boolean) {
    this.$cookies.remove('userData', { domain: this.config.tld });
    this.$cookies.remove('userAuthorized', { domain: this.config.tld });
    this.referrer = null;
    this.data.$isLoggedIn = false;
    this.data = null;
    // TODO: for some reason the state is not changing when logout, this is a quick fix
    if (!noReload) {
      window.location.href = window.location.origin;
    }
  };

  forgotPassword(email: string) {
    const params = {
      format: 'json',
      track: false,
      includeCookie: true,
      ugcCounts: true,
      email: email
    };

    const url = `${this.config.apiRoot}login/sendResetPasswordEmail`;
    return this.$http.post(url, {}, { params });
  }

  private isValidEmail(email: string) {
    const regExp = /^[-0-9a-zA-Z.+_]+@[-0-9a-zA-Z.+_]+\.[a-zA-Z]{2,4}$/;
    return regExp.test(email);
  }
}
